-- Local library instances
local XToLevel = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0")
local waterfall = AceLibrary("Waterfall-1.0")

-- Slash command table
local cfgSlashOptions = { 
    type = 'group',
    args = {
        config = {
            type = 'execute',
            name = 'config',
            desc = 'Open the config GUI',
			func = "Config_Open"
        },
		status = {
            type = 'execute',
            name = 'status',
            desc = 'Show the current addon settings',
			func = "Config_Status"
        }
    }
}
-- Per-character settings
chatEnabled = false;
floatingEnabled = true;

-- XPToLevel On Enable event
--  Initializes everything
function XToLevel:OnEnable()
    self:RegisterEvent("CHAT_MSG_COMBAT_XP_GAIN")
    self:RegisterChatCommand("/xtl", "/xtolevel", cfgSlashOptions)
    
    ConfigWindow_Init()
end

-- Register the Config Window watefall instance
function ConfigWindow_Init()
	waterfall:Register('ConfigWindow',
		'title','XToLevel Config',
		'colorR', 0.0,
		'colorG', 0.5,
		'colroB', 0.25,
		'tree', 
		{
			{
				text = 'Display',
				id = 'display',
				isOpen = true,
				isSelected = true
			}
		},
	 'children', 
		function(id)
			if id=='display' then
				waterfall:AddControl(
					'type', 'checkbox',
					'text', "Show in chat window",
					'checked', chatEnabled,
					'fullRefresh', true,
					'setFunc', function() chatEnabled = not chatEnabled end
		        )
				waterfall:AddControl(
					'type', 'checkbox',
					'text', "Show floating message",
					'checked', floatingEnabled,
					'fullRefresh', true,
					'setFunc', function() floatingEnabled = not floatingEnabled end
		        )
				return 'Display Config'
			end
		end 
	)
end

function XToLevel:Config_Open()
	waterfall:Open('ConfigWindow', 'display')
end

function XToLevel:Config_Status()
	self:Print("Chat message: ", showChatMsg)
	self:Print("Floating message: ", showFloatingMsg)
end

function XToLevel:CHAT_MSG_COMBAT_XP_GAIN()
	if (event=="CHAT_MSG_COMBAT_XP_GAIN") then
		-- Check if no message should be sent
		if (not chatEnabled and  not floatingEnabled) then
			return
		end
	
		local msg = "n/a"
			
		-- Gather the XP data
		local playerXP = UnitXP("player")
		local playerXPMax = UnitXPMax("player")
		local playerXPRested = GetXPExhaustion() or 0
		
		-- Check if the rested bonus extends beiond the next level
		if(playerXP + playerXPRested > playerXPMax) then
			playerXPRested = (playerXPMax - playerXP)
		end
		
		-- Extract the XP from the message
		local gStart, gEnd = string.find(arg1, "gain")
		local eStart, eEnd = string.find(arg1, "experience")
		local xpGained = tonumber(strsub(arg1, gEnd + 2, eStart - 2))
	
		-- Level reached
		if(playerXP + xpGained > playerXPMax) then
			msg = "Level reached. Congratulations"
			
		-- Quest turn-in
		elseif(nil == string.find(arg1, "dies")) then
			-- Calculate the ammount of quest turnins to level
			--  (TotalLevelXP - XPSoFar - GainedXP) / GainedXP
			local questsToLevel = ceil((playerXPMax - playerXP - xpGained) / xpGained)
			
			-- Create message
			msg = questsToLevel .. " more quests like that needed to level"
			
		-- Mob kill
		else
			-- Load mob name
			local dStart, dEnd = string.find(arg1, "dies")
			local mobName = strsub(arg1, 0, dStart - 2)
			
			-- Check if the XP gained was rested
			if(playerXPRested > 0) then
				if(playerXPRested * 2 < xpGained) then
					xpGained = xpGained - playerXPRested
				else
					xpGained = xpGained / 2
				end
			end
			
			-- Calculate the ammount of mobs needed
			--  (TotalLevelXP - XPSoFar - (RestedBonus)) / GainedXP
			local mobsNeededToLevel = ceil((playerXPMax - playerXP - (playerXPRested / 2) - xpGained) / xpGained)
			
			-- Create Message
			msg = mobsNeededToLevel .." ".. mobName .." needed to level"
		end
			
		-- Send Messages
		if chatEnabled then
			DEFAULT_CHAT_FRAME:AddMessage(msg, 0, 100, 200, 55, 4);
		end
		if floatingEnabled then
			UIErrorsFrame:AddMessage(msg, 0, 100, 200, 55, 4);
		end
	end
end